<?php
/**
 * Twig Extension Class: for handling the filesystem of Twig
 */
namespace EwTools\Twig;

use RecursiveDirectoryIterator;
use RecursiveFilterIterator;
use RecursiveIterator;
use RecursiveIteratorIterator;
use Twig_Error;
use Twig_Loader_Chain;
use Twig_Loader_Filesystem;

/**
 * Every_Loader
 *
 * @link    http://infomaker.se
 * @package EwTools\Twig
 * @since   EwTools\Twig\Every_Loader 0.1
 */
class Every_Loader extends Twig_Loader_Filesystem {
	
	/**
	 * Every_Loader constructor.
	 *
	 * @since 0.1
	 *
	 * @param array $paths
	 */
	function __construct( $paths = [] ) {
		parent::__construct( $paths );
	}
	
	/**
	 * Adds the folders containing the files to be loaded
	 *
	 * @param string $folder_path
	 * @param string $namespace
	 * @param array  $exclude
	 *
	 * @throws Twig_Error
	 * @since 0.1
	 * @return void
	 */
	public function addPathAndSubfolders( $folder_path, $namespace = Twig_Loader_Filesystem::MAIN_NAMESPACE, $exclude = [] ) {
		if ( ! file_exists( $folder_path ) ) {
			throw new Twig_Error( $folder_path . " does not exist" );
		}
		
		$dir      = new RecursiveDirectoryIterator( $folder_path );
		$filtered = new DirFilter( $dir, $exclude );
		
		foreach ( new RecursiveIteratorIterator( $filtered ) as $d ) {
			$this->addPath( $d, $namespace );
		}
	}
	
	/**
     * No documentation yet
	 * @todo Add documentation
	 *
	 * @param array $array
	 *
	 * @since 0.1
	 * @return Twig_Loader_Chain
	 */
	public function getChain( array $array ) {
		return new Twig_Loader_Chain( $array );
	}
}

/**
 * DirFilter
 * @todo Move to own file? Where is it used?
 *
 * @link    http://infomaker.se
 * @package EwTools\Twig
 * @since   EwTools\Twig\DirFilter 0.1
 */
class DirFilter extends RecursiveFilterIterator {
	
	/**
	 * Contains an array of files and folders to exclude
	 *
	 * @var array
	 */
	protected $exclude;
	
	/**
	 * DirFilter constructor.
	 *
	 * @since 0.1
	 *
	 * @param \RecursiveIterator $iterator
	 * @param array              $exclude
	 */
	public function __construct( RecursiveIterator $iterator, array $exclude ) {
		parent::__construct( $iterator );
		$this->exclude = $exclude;
	}
	
	/**
     * No documentation yet
	 * @todo Add documentation
	 *
	 * @since 0.1
	 * @return bool
	 */
	public function accept() {
		return ( $this->isDir() && ! in_array( $this->getFilename(), $this->exclude ) );
	}
	
	/**
     * No documentation yet
	 * @todo Add documentation
	 *
	 * @since 0.1
	 * @return DirFilter
	 */
	public function getChildren() {
		return new DirFilter( $this->getInnerIterator()->getChildren(), $this->exclude );
	}
}